"use strict";
/// <reference path="../../../node_modules/@types/jquery/misc.d.ts" />
/// <reference path="../../../node_modules/@types/jquery/JQueryStatic.d.ts" />
Object.defineProperty(exports, "__esModule", { value: true });
exports.JQueryODataClient = exports.getV2OrV4ErrorMessage = void 0;
const tslib_1 = require("tslib");
const AjaxConfig_1 = require("./AjaxConfig");
const JQueryODataClientError_1 = require("./JQueryODataClientError");
const getV2OrV4ErrorMessage = (errorResponse) => {
    var _a;
    const eMsg = (_a = errorResponse === null || errorResponse === void 0 ? void 0 : errorResponse.error) === null || _a === void 0 ? void 0 : _a.message;
    return typeof (eMsg === null || eMsg === void 0 ? void 0 : eMsg.value) === "string" ? eMsg.value : eMsg;
};
exports.getV2OrV4ErrorMessage = getV2OrV4ErrorMessage;
class JQueryODataClient {
    constructor(jquery, config, clientOptions) {
        this.clientOptions = clientOptions;
        this.getErrorMessage = exports.getV2OrV4ErrorMessage;
        this.client = jquery;
        this.config = (0, AjaxConfig_1.getDefaultConfig)(config);
    }
    setErrorMessageRetriever(getErrorMsg) {
        this.getErrorMessage = getErrorMsg;
    }
    setupSecurityToken() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.csrfToken == null) {
                this.csrfToken = yield this.fetchSecurityToken();
            }
        });
    }
    fetchSecurityToken() {
        var _a, _b;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const fetchUrl = (_b = (_a = this.clientOptions) === null || _a === void 0 ? void 0 : _a.csrfTokenFetchUrl) !== null && _b !== void 0 ? _b : "/";
            const response = yield this.get(fetchUrl, { headers: { "x-csrf-token": "Fetch" } });
            return response.headers["x-csrf-token"];
        });
    }
    sendRequest(config, requestConfig) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const mergedConfig = (0, AjaxConfig_1.mergeAjaxConfig)((0, AjaxConfig_1.mergeAjaxConfig)(this.config, requestConfig), config);
            // setup automatic CSRF token handling
            if (((_a = this.clientOptions) === null || _a === void 0 ? void 0 : _a.useCsrfProtection) &&
                mergedConfig.method &&
                ["POST", "PUT", "PATCH", "DELETE"].includes(mergedConfig.method.toUpperCase())) {
                yield this.setupSecurityToken();
                if (!mergedConfig.headers) {
                    mergedConfig.headers = {};
                }
                if (this.csrfToken) {
                    mergedConfig.headers["x-csrf-token"] = this.csrfToken;
                }
            }
            // the actual request
            return new Promise((resolve, reject) => {
                this.client.ajax(Object.assign(Object.assign({}, mergedConfig), { success: (response, textStatus, jqXHR) => {
                        resolve({
                            status: jqXHR.status,
                            statusText: jqXHR.statusText,
                            headers: jqXHR.getAllResponseHeaders(),
                            data: response,
                        });
                    }, error: (jqXHR, textStatus, thrownError) => {
                        var _a;
                        // automatic CSRF token handling
                        if (((_a = this.clientOptions) === null || _a === void 0 ? void 0 : _a.useCsrfProtection) &&
                            jqXHR.status === 403 &&
                            jqXHR.getResponseHeader("x-csrf-token") === "Required") {
                            // csrf token expired, let's reset it and perform the original request again
                            this.csrfToken = undefined;
                            this.sendRequest(config, requestConfig).then(resolve).catch(reject);
                        }
                        // actual error handling
                        else {
                            const message = this.getErrorMessage(jqXHR.responseJSON);
                            if (message) {
                                reject(new JQueryODataClientError_1.JQueryODataClientError("Server responded with error: " + message, jqXHR.status, { cause: jqXHR }));
                            }
                            else {
                                reject(new JQueryODataClientError_1.JQueryODataClientError(textStatus + " " + thrownError, jqXHR.status, { cause: jqXHR }));
                            }
                        }
                    } }));
            });
        });
    }
    post(url, data, requestConfig) {
        return this.sendRequest({ url, data, method: "POST" }, requestConfig);
    }
    get(url, requestConfig) {
        return this.sendRequest({ url, method: "GET" }, requestConfig);
    }
    put(url, data, requestConfig) {
        return this.sendRequest({ url, data, method: "PUT" }, requestConfig);
    }
    patch(url, data, requestConfig) {
        return this.sendRequest({ url, data, method: "PATCH" }, requestConfig);
    }
    merge(url, data, requestConfig) {
        const config = (0, AjaxConfig_1.mergeAjaxConfig)(requestConfig, {
            headers: {
                "X-Http-Method": "MERGE",
            },
        });
        return this.sendRequest({ url, data, method: "POST" }, config);
    }
    delete(url, requestConfig) {
        return this.sendRequest({ url, method: "DELETE" }, requestConfig);
    }
}
exports.JQueryODataClient = JQueryODataClient;
//# sourceMappingURL=JQueryODataClient.js.map